@@ -56,6 +56,7 @@ from .basic import (
     PCAN_CHANNEL_FEATURES,
     FEATURE_FD_CAPABLE,
     PCAN_DICT_STATUS,
+    PCAN_BUSOFF_AUTORESET,
 )
 
 
@@ -103,6 +104,7 @@ class PcanBus(BusABC):
     def __init__(
         self,
         channel="PCAN_USBBUS1",
+        device_id=None,
         state=BusState.ACTIVE,
         bitrate=500000,
         *args,
@@ -111,14 +113,22 @@ class PcanBus(BusABC):
         """A PCAN USB interface to CAN.
 
         On top of the usual :class:`~can.Bus` methods provided,
-        the PCAN interface includes the :meth:`~can.interface.pcan.PcanBus.flash`
-        and :meth:`~can.interface.pcan.PcanBus.status` methods.
+        the PCAN interface includes the :meth:`flash`
+        and :meth:`status` methods.
 
         :param str channel:
             The can interface name. An example would be 'PCAN_USBBUS1'.
             Alternatively the value can be an int with the numerical value.
             Default is 'PCAN_USBBUS1'
 
+        :param int device_id:
+            Select the PCAN interface based on its ID. The device ID is a 8/32bit
+            value that can be configured for each PCAN device. If you set the
+            device_id parameter, it takes precedence over the channel parameter.
+            The constructor searches all connected interfaces and initializes the
+            first one that matches the parameter value. If no device is found,
+            an exception is raised.
+
         :param can.bus.BusState state:
             BusState of the channel.
             Default is ACTIVE
@@ -197,7 +207,20 @@ class PcanBus(BusABC):
             In the range (1..16).
             Ignored if not using CAN-FD.
 
+        :param bool auto_reset:
+            Enable automatic recovery in bus off scenario.
+            Resetting the driver takes ~500ms during which
+            it will not be responsive.
         """
+        self.m_objPCANBasic = PCANBasic()
+
+        if device_id is not None:
+            channel = self._find_channel_by_dev_id(device_id)
+
+            if channel is None:
+                err_msg = "Cannot find a channel with ID {:08x}".format(device_id)
+                raise ValueError(err_msg)
+
         self.channel_info = str(channel)
         self.fd = kwargs.get("fd", False)
         pcan_bitrate = PCAN_BITRATES.get(bitrate, PCAN_BAUD_500K)
@@ -209,7 +232,6 @@ class PcanBus(BusABC):
         if not isinstance(channel, int):
             channel = PCAN_CHANNEL_NAMES[channel]
 
-        self.m_objPCANBasic = PCANBasic()
         self.m_PcanHandle = channel
 
         self.check_api_version()
@@ -259,6 +281,14 @@ class PcanBus(BusABC):
                     "Ignoring error. PCAN_ALLOW_ERROR_FRAMES is still unsupported by OSX Library PCANUSB v0.10"
                 )
 
+        if kwargs.get("auto_reset", False):
+            result = self.m_objPCANBasic.SetValue(
+                self.m_PcanHandle, PCAN_BUSOFF_AUTORESET, PCAN_PARAMETER_ON
+            )
+
+            if result != PCAN_ERROR_OK:
+                raise PcanCanInitializationError(self._get_formatted_error(result))
+
         if HAS_EVENTS:
             self._recv_event = CreateEvent(None, 0, 0, None)
             result = self.m_objPCANBasic.SetValue(
@@ -269,6 +299,28 @@ class PcanBus(BusABC):
 
         super().__init__(channel=channel, state=state, bitrate=bitrate, *args, **kwargs)
 
+    def _find_channel_by_dev_id(self, device_id):
+        """
+        Iterate over all possible channels to find a channel that matches the device
+        ID. This method is somewhat brute force, but the Basic API only offers a
+        suitable API call since V4.4.0.
+
+        :param device_id: The device_id for which to search for
+        :return: The name of a PCAN channel that matches the device ID, or None if
+            no channel can be found.
+        """
+        for ch_name, ch_handle in PCAN_CHANNEL_NAMES.items():
+            err, cur_dev_id = self.m_objPCANBasic.GetValue(
+                ch_handle, PCAN_DEVICE_NUMBER
+            )
+            if err != PCAN_ERROR_OK:
+                continue
+
+            if cur_dev_id == device_id:
+                return ch_name
+
+        return None
+
     def _get_formatted_error(self, error):
         """
         Gets the text using the GetErrorText API function.
