@@ -2,11 +2,13 @@
 Interface for Chinese Robotell compatible interfaces (win32/linux).
 """
 
+import io
 import time
 import logging
+from typing import Optional
 
 from can import BusABC, Message
-from ..exceptions import CanInterfaceNotImplementedError
+from ..exceptions import CanInterfaceNotImplementedError, CanOperationError
 
 logger = logging.getLogger(__name__)
 
@@ -367,17 +369,20 @@ class robotellBus(BusABC):
         self.serialPortOrig.close()
 
     def fileno(self):
-        if hasattr(self.serialPortOrig, "fileno"):
+        try:
             return self.serialPortOrig.fileno()
-        # Return an invalid file descriptor on Windows
-        return -1
+        except io.UnsupportedOperation:
+            raise NotImplementedError(
+                "fileno is not implemented using current CAN bus on this platform"
+            )
+        except Exception as exception:
+            raise CanOperationError("Cannot fetch fileno") from exception
 
-    def get_serial_number(self, timeout):
+    def get_serial_number(self, timeout: Optional[int]) -> Optional[str]:
         """Get serial number of the slcan interface.
-        :type timeout: int or None
+
         :param timeout:
             seconds to wait for serial number or None to wait indefinitely
-        :rtype str or None
         :return:
             None on timeout or a str object.
         """
