@@ -349,7 +349,9 @@ class CyclicSendTask(
         self.task_id = task_id
         self._tx_setup(self.messages)
 
-    def _tx_setup(self, messages: Sequence[Message]) -> None:
+    def _tx_setup(
+        self, messages: Sequence[Message], raise_if_task_exists: bool = True
+    ) -> None:
         # Create a low level packed frame to pass to the kernel
         body = bytearray()
         self.flags = CAN_FD_FRAME if messages[0].is_fd else 0
@@ -363,7 +365,8 @@ class CyclicSendTask(
             ival1 = 0.0
             ival2 = self.period
 
-        self._check_bcm_task()
+        if raise_if_task_exists:
+            self._check_bcm_task()
 
         header = build_bcm_transmit_header(
             self.task_id, count, ival1, ival2, self.flags, nframes=len(messages)
@@ -375,7 +378,7 @@ class CyclicSendTask(
 
     def _check_bcm_task(self) -> None:
         # Do a TX_READ on a task ID, and check if we get EINVAL. If so,
-        # then we are referring to a CAN message with the existing ID
+        # then we are referring to a CAN message with an existing ID
         check_header = build_bcm_header(
             opcode=CAN_BCM_TX_READ,
             flags=0,
@@ -387,12 +390,19 @@ class CyclicSendTask(
             can_id=self.task_id,
             nframes=0,
         )
+        log.debug(
+            f"Reading properties of (cyclic) transmission task id={self.task_id}",
+        )
         try:
             self.bcm_socket.send(check_header)
         except OSError as error:
             if error.errno != errno.EINVAL:
                 raise can.CanOperationError("failed to check", error.errno) from error
+            else:
+                log.debug("Invalid argument - transmission task not known to kernel")
         else:
+            # No exception raised - transmission task with this ID exists in kernel.
+            # Existence of an existing transmission task might not be a problem!
             raise can.CanOperationError(
                 f"A periodic task for task ID {self.task_id} is already in progress "
                 "by the SocketCAN Linux layer"
@@ -402,7 +412,7 @@ class CyclicSendTask(
         """Stop a task by sending TX_DELETE message to Linux kernel.
 
         This will delete the entry for the transmission of the CAN-message
-        with the specified :attr:`~task_id` identifier. The message length
+        with the specified ``task_id`` identifier. The message length
         for the command TX_DELETE is {[bcm_msg_head]} (only the header).
         """
         log.debug("Stopping periodic task")
@@ -438,15 +448,15 @@ class CyclicSendTask(
         send_bcm(self.bcm_socket, header + body)
 
     def start(self) -> None:
-        """Start a periodic task by sending TX_SETUP message to Linux kernel.
+        """Restart a periodic task by sending TX_SETUP message to Linux kernel.
 
         It verifies presence of the particular BCM task through sending TX_READ
         message to Linux kernel prior to scheduling.
 
         :raises ValueError:
-            If the task referenced by :attr:`~task_id` is already running.
+            If the task referenced by ``task_id`` is already running.
         """
-        self._tx_setup(self.messages)
+        self._tx_setup(self.messages, raise_if_task_exists=False)
 
 
 class MultiRateCyclicSendTask(CyclicSendTask):
@@ -530,7 +540,7 @@ def capture_message(
             channel = addr[0] if isinstance(addr, tuple) else addr
         else:
             channel = None
-    except socket.error as error:
+    except OSError as error:
         raise can.CanOperationError(f"Error receiving: {error.strerror}", error.errno)
 
     can_id, can_dlc, flags, data = dissect_can_frame(cf)
@@ -617,9 +627,10 @@ class SocketcanBus(BusABC):
 
         If setting some socket options fails, an error will be printed but no exception will be thrown.
         This includes enabling:
-         - that own messages should be received,
-         - CAN-FD frames and
-         - error frames.
+
+            - that own messages should be received,
+            - CAN-FD frames and
+            - error frames.
 
         :param channel:
             The can interface name with which to create this bus.
@@ -656,7 +667,7 @@ class SocketcanBus(BusABC):
             self.socket.setsockopt(
                 SOL_CAN_RAW, CAN_RAW_LOOPBACK, 1 if local_loopback else 0
             )
-        except socket.error as error:
+        except OSError as error:
             log.error("Could not set local loopback flag(%s)", error)
 
         # set the receive_own_messages parameter
@@ -664,21 +675,21 @@ class SocketcanBus(BusABC):
             self.socket.setsockopt(
                 SOL_CAN_RAW, CAN_RAW_RECV_OWN_MSGS, 1 if receive_own_messages else 0
             )
-        except socket.error as error:
+        except OSError as error:
             log.error("Could not receive own messages (%s)", error)
 
         # enable CAN-FD frames if desired
         if fd:
             try:
                 self.socket.setsockopt(SOL_CAN_RAW, CAN_RAW_FD_FRAMES, 1)
-            except socket.error as error:
+            except OSError as error:
                 log.error("Could not enable CAN-FD frames (%s)", error)
 
         if not ignore_rx_error_frames:
             # enable error frames
             try:
                 self.socket.setsockopt(SOL_CAN_RAW, CAN_RAW_ERR_FILTER, 0x1FFFFFFF)
-            except socket.error as error:
+            except OSError as error:
                 log.error("Could not enable error frames (%s)", error)
 
         # enable nanosecond resolution timestamping
@@ -714,7 +725,7 @@ class SocketcanBus(BusABC):
             # get all sockets that are ready (can be a list with a single value
             # being self.socket or an empty list if self.socket is not ready)
             ready_receive_sockets, _, _ = select.select([self.socket], [], [], timeout)
-        except socket.error as error:
+        except OSError as error:
             # something bad happened (e.g. the interface went down)
             raise can.CanOperationError(
                 f"Failed to receive: {error.strerror}", error.errno
@@ -739,7 +750,7 @@ class SocketcanBus(BusABC):
             Wait up to this many seconds for the transmit queue to be ready.
             If not given, the call may fail immediately.
 
-        :raises can.CanError:
+        :raises ~can.exceptions.CanError:
             if the message could not be written.
         """
         log.debug("We've been asked to write a message to the bus")
@@ -776,7 +787,7 @@ class SocketcanBus(BusABC):
                 sent = self.socket.sendto(data, (channel,))
             else:
                 sent = self.socket.send(data)
-        except socket.error as error:
+        except OSError as error:
             raise can.CanOperationError(
                 f"Failed to transmit: {error.strerror}", error.errno
             )
@@ -829,7 +840,7 @@ class SocketcanBus(BusABC):
 
     def _get_next_task_id(self) -> int:
         with self._task_id_guard:
-            self._task_id = (self._task_id + 1) % (2 ** 32 - 1)
+            self._task_id = (self._task_id + 1) % (2**32 - 1)
             return self._task_id
 
     def _get_bcm_socket(self, channel: str) -> socket.socket:
@@ -840,7 +851,7 @@ class SocketcanBus(BusABC):
     def _apply_filters(self, filters: Optional[can.typechecking.CanFilters]) -> None:
         try:
             self.socket.setsockopt(SOL_CAN_RAW, CAN_RAW_FILTER, pack_filters(filters))
-        except socket.error as error:
+        except OSError as error:
             # fall back to "software filtering" (= not in kernel)
             self._is_filtered = False
             log.error(
