@@ -21,13 +21,14 @@ from typing import (
     Any,
     Dict,
     Callable,
+    cast,
 )
 
 WaitForSingleObject: Optional[Callable[[int, int], int]]
 INFINITE: Optional[int]
 try:
     # Try builtin Python 3 Windows API
-    from _winapi import WaitForSingleObject, INFINITE
+    from _winapi import WaitForSingleObject, INFINITE  # type: ignore
 
     HAS_EVENTS = True
 except ImportError:
@@ -43,7 +44,7 @@ from can.util import (
     deprecated_args_alias,
     time_perfcounter_correlation,
 )
-from can.typechecking import AutoDetectedConfig, CanFilters, Channel
+from can.typechecking import AutoDetectedConfig, CanFilters
 
 # Define Module Logger
 # ====================
@@ -82,7 +83,7 @@ class VectorBus(BusABC):
         poll_interval: float = 0.01,
         receive_own_messages: bool = False,
         bitrate: Optional[int] = None,
-        rx_queue_size: int = 2 ** 14,
+        rx_queue_size: int = 2**14,
         app_name: Optional[str] = "CANalyzer",
         serial: Optional[int] = None,
         fd: bool = False,
@@ -137,11 +138,11 @@ class VectorBus(BusABC):
         :param tseg2_dbr:
             Bus timing value tseg2 (data)
 
-        :raise can.CanInterfaceNotImplementedError:
+        :raise ~can.exceptions.CanInterfaceNotImplementedError:
             If the current operating system is not supported or the driver could not be loaded.
-        :raise can.CanInitializationError:
+        :raise ~can.exceptions.CanInitializationError:
             If the bus could not be set up.
-            This may or may not be a :class:`can.interfaces.vector.VectorInitializationError`.
+            This may or may not be a :class:`~can.interfaces.vector.VectorInitializationError`.
         """
         if os.name != "nt" and not kwargs.get("_testing", False):
             raise CanInterfaceNotImplementedError(
@@ -152,6 +153,7 @@ class VectorBus(BusABC):
         if xldriver is None:
             raise CanInterfaceNotImplementedError("The Vector API has not been loaded")
         self.xldriver = xldriver  # keep reference so mypy knows it is not None
+        self.xldriver.xlOpenDriver()
 
         self.poll_interval = poll_interval
 
@@ -165,7 +167,7 @@ class VectorBus(BusABC):
             self.channels = [int(ch) for ch in channel]
         else:
             raise TypeError(
-                f"Invalid type for channels parameter: {type(channel).__name__}"
+                f"Invalid type for parameter 'channel': {type(channel).__name__}"
             )
 
         self._app_name = app_name.encode() if app_name is not None else b""
@@ -174,136 +176,118 @@ class VectorBus(BusABC):
             ", ".join(f"CAN {ch + 1}" for ch in self.channels),
         )
 
-        if serial is not None:
-            app_name = None
-            channel_index = []
-            channel_configs = get_channel_configs()
-            for channel_config in channel_configs:
-                if channel_config.serialNumber == serial:
-                    if channel_config.hwChannel in self.channels:
-                        channel_index.append(channel_config.channelIndex)
-            if channel_index:
-                if len(channel_index) != len(self.channels):
-                    LOG.info(
-                        "At least one defined channel wasn't found on the specified hardware."
-                    )
-                self.channels = channel_index
-            else:
-                # Is there any better way to raise the error?
-                raise CanInitializationError(
-                    "None of the configured channels could be found on the specified hardware."
-                )
+        channel_configs = get_channel_configs()
 
-        self.xldriver.xlOpenDriver()
-        self.port_handle = xlclass.XLportHandle(xldefine.XL_INVALID_PORTHANDLE)
         self.mask = 0
         self.fd = fd
-        # Get channels masks
-        self.channel_masks: Dict[Optional[Channel], int] = {}
-        self.index_to_channel = {}
+        self.channel_masks: Dict[int, int] = {}
+        self.index_to_channel: Dict[int, int] = {}
 
         for channel in self.channels:
-            if app_name:
-                # Get global channel index from application channel
-                hw_type, hw_index, hw_channel = self.get_application_config(
-                    app_name, channel
-                )
-                LOG.debug("Channel index %d found", channel)
-                idx = self.xldriver.xlGetChannelIndex(hw_type, hw_index, hw_channel)
-                if idx < 0:
-                    # Undocumented behavior! See issue #353.
-                    # If hardware is unavailable, this function returns -1.
-                    # Raise an exception as if the driver
-                    # would have signalled XL_ERR_HW_NOT_PRESENT.
-                    raise VectorInitializationError(
-                        xldefine.XL_Status.XL_ERR_HW_NOT_PRESENT,
-                        xldefine.XL_Status.XL_ERR_HW_NOT_PRESENT.name,
-                        "xlGetChannelIndex",
-                    )
-            else:
-                # Channel already given as global channel
-                idx = channel
-            mask = 1 << idx
-            self.channel_masks[channel] = mask
-            self.index_to_channel[idx] = channel
-            self.mask |= mask
+            channel_index = self._find_global_channel_idx(
+                channel=channel,
+                serial=serial,
+                app_name=app_name,
+                channel_configs=channel_configs,
+            )
+            LOG.debug("Channel index %d found", channel)
+
+            channel_mask = 1 << channel_index
+            self.channel_masks[channel] = channel_mask
+            self.index_to_channel[channel_index] = channel
+            self.mask |= channel_mask
 
         permission_mask = xlclass.XLaccess()
         # Set mask to request channel init permission if needed
         if bitrate or fd:
             permission_mask.value = self.mask
-        if fd:
-            self.xldriver.xlOpenPort(
-                self.port_handle,
-                self._app_name,
-                self.mask,
-                permission_mask,
-                rx_queue_size,
-                xldefine.XL_InterfaceVersion.XL_INTERFACE_VERSION_V4,
-                xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
-            )
-        else:
-            self.xldriver.xlOpenPort(
-                self.port_handle,
-                self._app_name,
-                self.mask,
-                permission_mask,
-                rx_queue_size,
-                xldefine.XL_InterfaceVersion.XL_INTERFACE_VERSION,
-                xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
-            )
+
+        interface_version = (
+            xldefine.XL_InterfaceVersion.XL_INTERFACE_VERSION_V4
+            if fd
+            else xldefine.XL_InterfaceVersion.XL_INTERFACE_VERSION
+        )
+
+        self.port_handle = xlclass.XLportHandle(xldefine.XL_INVALID_PORTHANDLE)
+        self.xldriver.xlOpenPort(
+            self.port_handle,
+            self._app_name,
+            self.mask,
+            permission_mask,
+            rx_queue_size,
+            interface_version,
+            xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
+        )
+        self.permission_mask = permission_mask.value
+
         LOG.debug(
             "Open Port: PortHandle: %d, PermissionMask: 0x%X",
             self.port_handle.value,
             permission_mask.value,
         )
 
-        if permission_mask.value == self.mask:
+        # set CAN settings
+        for channel in self.channels:
+            if self._has_init_access(channel):
+                if fd:
+                    self._set_bitrate_canfd(
+                        channel=channel,
+                        bitrate=bitrate,
+                        data_bitrate=data_bitrate,
+                        sjw_abr=sjw_abr,
+                        tseg1_abr=tseg1_abr,
+                        tseg2_abr=tseg2_abr,
+                        sjw_dbr=sjw_dbr,
+                        tseg1_dbr=tseg1_dbr,
+                        tseg2_dbr=tseg2_dbr,
+                    )
+                elif bitrate:
+                    self._set_bitrate_can(channel=channel, bitrate=bitrate)
+
+        # Check CAN settings
+        for channel in self.channels:
+            if kwargs.get("_testing", False):
+                # avoid check if xldriver is mocked for testing
+                break
+
+            bus_params = self._read_bus_params(channel)
             if fd:
-                self.canFdConf = xlclass.XLcanFdConf()
-                if bitrate:
-                    self.canFdConf.arbitrationBitRate = int(bitrate)
-                else:
-                    self.canFdConf.arbitrationBitRate = 500000
-                self.canFdConf.sjwAbr = int(sjw_abr)
-                self.canFdConf.tseg1Abr = int(tseg1_abr)
-                self.canFdConf.tseg2Abr = int(tseg2_abr)
-                if data_bitrate:
-                    self.canFdConf.dataBitRate = int(data_bitrate)
-                else:
-                    self.canFdConf.dataBitRate = self.canFdConf.arbitrationBitRate
-                self.canFdConf.sjwDbr = int(sjw_dbr)
-                self.canFdConf.tseg1Dbr = int(tseg1_dbr)
-                self.canFdConf.tseg2Dbr = int(tseg2_dbr)
-
-                self.xldriver.xlCanFdSetConfiguration(
-                    self.port_handle, self.mask, self.canFdConf
-                )
-                LOG.info(
-                    "SetFdConfig.: ABaudr.=%u, DBaudr.=%u",
-                    self.canFdConf.arbitrationBitRate,
-                    self.canFdConf.dataBitRate,
-                )
-                LOG.info(
-                    "SetFdConfig.: sjwAbr=%u, tseg1Abr=%u, tseg2Abr=%u",
-                    self.canFdConf.sjwAbr,
-                    self.canFdConf.tseg1Abr,
-                    self.canFdConf.tseg2Abr,
-                )
-                LOG.info(
-                    "SetFdConfig.: sjwDbr=%u, tseg1Dbr=%u, tseg2Dbr=%u",
-                    self.canFdConf.sjwDbr,
-                    self.canFdConf.tseg1Dbr,
-                    self.canFdConf.tseg2Dbr,
-                )
+                _canfd = bus_params.canfd
+                if not all(
+                    [
+                        bus_params.bus_type is xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
+                        _canfd.can_op_mode
+                        & xldefine.XL_CANFD_BusParams_CanOpMode.XL_BUS_PARAMS_CANOPMODE_CANFD,
+                        _canfd.bitrate == bitrate if bitrate else True,
+                        _canfd.sjw_abr == sjw_abr if bitrate else True,
+                        _canfd.tseg1_abr == tseg1_abr if bitrate else True,
+                        _canfd.tseg2_abr == tseg2_abr if bitrate else True,
+                        _canfd.data_bitrate == data_bitrate if data_bitrate else True,
+                        _canfd.sjw_dbr == sjw_dbr if data_bitrate else True,
+                        _canfd.tseg1_dbr == tseg1_dbr if data_bitrate else True,
+                        _canfd.tseg2_dbr == tseg2_dbr if data_bitrate else True,
+                    ]
+                ):
+                    raise CanInitializationError(
+                        f"The requested CAN FD settings could not be set for channel {channel}. "
+                        f"Another application might have set incompatible settings. "
+                        f"These are the currently active settings: {_canfd._asdict()}"
+                    )
             else:
-                if bitrate:
-                    self.xldriver.xlCanSetChannelBitrate(
-                        self.port_handle, permission_mask, bitrate
+                _can = bus_params.can
+                if not all(
+                    [
+                        bus_params.bus_type is xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
+                        _can.can_op_mode
+                        & xldefine.XL_CANFD_BusParams_CanOpMode.XL_BUS_PARAMS_CANOPMODE_CAN20,
+                        _can.bitrate == bitrate if bitrate else True,
+                    ]
+                ):
+                    raise CanInitializationError(
+                        f"The requested CAN settings could not be set for channel {channel}. "
+                        f"Another application might have set incompatible settings. "
+                        f"These are the currently active settings: {_can._asdict()}"
                     )
-                    LOG.info("SetChannelBitrate: baudr.=%u", bitrate)
-        else:
-            LOG.info("No init access!")
 
         # Enable/disable TX receipts
         tx_receipts = 1 if receive_own_messages else 0
@@ -348,6 +332,168 @@ class VectorBus(BusABC):
         self._is_filtered = False
         super().__init__(channel=channel, can_filters=can_filters, **kwargs)
 
+    def _find_global_channel_idx(
+        self,
+        channel: int,
+        serial: Optional[int],
+        app_name: Optional[str],
+        channel_configs: List["VectorChannelConfig"],
+    ) -> int:
+        if serial is not None:
+            hw_type: Optional[xldefine.XL_HardwareType] = None
+            for channel_config in channel_configs:
+                if channel_config.serial_number != serial:
+                    continue
+
+                hw_type = xldefine.XL_HardwareType(channel_config.hw_type)
+                if channel_config.hw_channel == channel:
+                    return channel_config.channel_index
+
+            if hw_type is None:
+                err_msg = f"No interface with serial {serial} found."
+            else:
+                err_msg = f"Channel {channel} not found on interface {hw_type.name} ({serial})."
+            raise CanInitializationError(
+                err_msg, error_code=xldefine.XL_Status.XL_ERR_HW_NOT_PRESENT
+            )
+
+        if app_name:
+            hw_type, hw_index, hw_channel = self.get_application_config(
+                app_name, channel
+            )
+            idx = cast(
+                int, self.xldriver.xlGetChannelIndex(hw_type, hw_index, hw_channel)
+            )
+            if idx < 0:
+                # Undocumented behavior! See issue #353.
+                # If hardware is unavailable, this function returns -1.
+                # Raise an exception as if the driver
+                # would have signalled XL_ERR_HW_NOT_PRESENT.
+                raise VectorInitializationError(
+                    xldefine.XL_Status.XL_ERR_HW_NOT_PRESENT,
+                    xldefine.XL_Status.XL_ERR_HW_NOT_PRESENT.name,
+                    "xlGetChannelIndex",
+                )
+            return idx
+
+        # check if channel is a valid global channel index
+        for channel_config in channel_configs:
+            if channel == channel_config.channel_index:
+                return channel
+
+        raise CanInitializationError(
+            f"Channel {channel} not found. The 'channel' parameter must be "
+            f"a valid global channel index if neither 'app_name' nor 'serial' were given.",
+            error_code=xldefine.XL_Status.XL_ERR_HW_NOT_PRESENT,
+        )
+
+    def _has_init_access(self, channel: int) -> bool:
+        return bool(self.permission_mask & self.channel_masks[channel])
+
+    def _read_bus_params(self, channel: int) -> "VectorBusParams":
+        channel_mask = self.channel_masks[channel]
+
+        vcc_list = get_channel_configs()
+        for vcc in vcc_list:
+            if vcc.channel_mask == channel_mask:
+                return vcc.bus_params
+
+        raise CanInitializationError(
+            f"Channel configuration for channel {channel} not found."
+        )
+
+    def _set_bitrate_can(
+        self,
+        channel: int,
+        bitrate: int,
+        sjw: Optional[int] = None,
+        tseg1: Optional[int] = None,
+        tseg2: Optional[int] = None,
+        sam: int = 1,
+    ) -> None:
+        kwargs = [sjw, tseg1, tseg2]
+        if any(kwargs) and not all(kwargs):
+            raise ValueError(
+                f"Either all of sjw, tseg1, tseg2 must be set or none of them."
+            )
+
+        # set parameters if channel has init access
+        if any(kwargs):
+            chip_params = xlclass.XLchipParams()
+            chip_params.bitRate = bitrate
+            chip_params.sjw = sjw
+            chip_params.tseg1 = tseg1
+            chip_params.tseg2 = tseg2
+            chip_params.sam = sam
+            self.xldriver.xlCanSetChannelParams(
+                self.port_handle,
+                self.channel_masks[channel],
+                chip_params,
+            )
+            LOG.info(
+                "xlCanSetChannelParams: baudr.=%u, sjwAbr=%u, tseg1Abr=%u, tseg2Abr=%u",
+                chip_params.bitRate,
+                chip_params.sjw,
+                chip_params.tseg1,
+                chip_params.tseg2,
+            )
+        else:
+            self.xldriver.xlCanSetChannelBitrate(
+                self.port_handle,
+                self.channel_masks[channel],
+                bitrate,
+            )
+            LOG.info("xlCanSetChannelBitrate: baudr.=%u", bitrate)
+
+    def _set_bitrate_canfd(
+        self,
+        channel: int,
+        bitrate: Optional[int] = None,
+        data_bitrate: Optional[int] = None,
+        sjw_abr: int = 2,
+        tseg1_abr: int = 6,
+        tseg2_abr: int = 3,
+        sjw_dbr: int = 2,
+        tseg1_dbr: int = 6,
+        tseg2_dbr: int = 3,
+    ) -> None:
+        # set parameters if channel has init access
+        canfd_conf = xlclass.XLcanFdConf()
+        if bitrate:
+            canfd_conf.arbitrationBitRate = int(bitrate)
+        else:
+            canfd_conf.arbitrationBitRate = 500_000
+        canfd_conf.sjwAbr = int(sjw_abr)
+        canfd_conf.tseg1Abr = int(tseg1_abr)
+        canfd_conf.tseg2Abr = int(tseg2_abr)
+        if data_bitrate:
+            canfd_conf.dataBitRate = int(data_bitrate)
+        else:
+            canfd_conf.dataBitRate = int(canfd_conf.arbitrationBitRate)
+        canfd_conf.sjwDbr = int(sjw_dbr)
+        canfd_conf.tseg1Dbr = int(tseg1_dbr)
+        canfd_conf.tseg2Dbr = int(tseg2_dbr)
+        self.xldriver.xlCanFdSetConfiguration(
+            self.port_handle, self.channel_masks[channel], canfd_conf
+        )
+        LOG.info(
+            "xlCanFdSetConfiguration.: ABaudr.=%u, DBaudr.=%u",
+            canfd_conf.arbitrationBitRate,
+            canfd_conf.dataBitRate,
+        )
+        LOG.info(
+            "xlCanFdSetConfiguration.: sjwAbr=%u, tseg1Abr=%u, tseg2Abr=%u",
+            canfd_conf.sjwAbr,
+            canfd_conf.tseg1Abr,
+            canfd_conf.tseg2Abr,
+        )
+        LOG.info(
+            "xlCanFdSetConfiguration.: sjwDbr=%u, tseg1Dbr=%u, tseg2Dbr=%u",
+            canfd_conf.sjwDbr,
+            canfd_conf.tseg1Dbr,
+            canfd_conf.tseg2Dbr,
+        )
+
     def _apply_filters(self, filters: Optional[CanFilters]) -> None:
         if filters:
             # Only up to one filter per ID type allowed
@@ -544,7 +690,7 @@ class VectorBus(BusABC):
 
     def _get_tx_channel_mask(self, msgs: Sequence[Message]) -> int:
         if len(msgs) == 1:
-            return self.channel_masks.get(msgs[0].channel, self.mask)
+            return self.channel_masks.get(msgs[0].channel, self.mask)  # type: ignore[arg-type]
         else:
             return self.mask
 
@@ -643,26 +789,28 @@ class VectorBus(BusABC):
         LOG.info("Found %d channels", len(channel_configs))
         for channel_config in channel_configs:
             if (
-                not channel_config.channelBusCapabilities
+                not channel_config.channel_bus_capabilities
                 & xldefine.XL_BusCapabilities.XL_BUS_ACTIVE_CAP_CAN
             ):
                 continue
             LOG.info(
-                "Channel index %d: %s", channel_config.channelIndex, channel_config.name
+                "Channel index %d: %s",
+                channel_config.channel_index,
+                channel_config.name,
             )
             configs.append(
                 {
                     # data for use in VectorBus.__init__():
                     "interface": "vector",
-                    "channel": channel_config.hwChannel,
-                    "serial": channel_config.serialNumber,
+                    "channel": channel_config.hw_channel,
+                    "serial": channel_config.serial_number,
                     # data for use in VectorBus.set_application_config():
-                    "hw_type": channel_config.hwType,
-                    "hw_index": channel_config.hwIndex,
-                    "hw_channel": channel_config.hwChannel,
+                    "hw_type": channel_config.hw_type,
+                    "hw_index": channel_config.hw_index,
+                    "hw_channel": channel_config.hw_channel,
                     # additional information:
                     "supports_fd": bool(
-                        channel_config.channelCapabilities
+                        channel_config.channel_capabilities
                         & xldefine.XL_ChannelCapabilities.XL_CHANNEL_FLAG_CANFD_ISO_SUPPORT
                     ),
                     "vector_channel_config": channel_config,
@@ -707,14 +855,24 @@ class VectorBus(BusABC):
         hw_channel = ctypes.c_uint()
         _app_channel = ctypes.c_uint(app_channel)
 
-        xldriver.xlGetApplConfig(
-            app_name.encode(),
-            _app_channel,
-            hw_type,
-            hw_index,
-            hw_channel,
-            xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
-        )
+        try:
+            xldriver.xlGetApplConfig(
+                app_name.encode(),
+                _app_channel,
+                hw_type,
+                hw_index,
+                hw_channel,
+                xldefine.XL_BusTypes.XL_BUS_TYPE_CAN,
+            )
+        except VectorError as e:
+            raise VectorInitializationError(
+                error_code=e.error_code,
+                error_string=(
+                    f"Vector HW Config: Channel '{app_channel}' of "
+                    f"application '{app_name}' is not assigned to any interface"
+                ),
+                function="xlGetApplConfig",
+            ) from None
         return xldefine.XL_HardwareType(hw_type.value), hw_index.value, hw_channel.value
 
     @staticmethod
@@ -781,54 +939,130 @@ class VectorBus(BusABC):
         self.xldriver.xlSetTimerRate(self.port_handle, timer_rate_10us)
 
 
+class VectorCanParams(NamedTuple):
+    bitrate: int
+    sjw: int
+    tseg1: int
+    tseg2: int
+    sam: int
+    output_mode: xldefine.XL_OutputMode
+    can_op_mode: xldefine.XL_CANFD_BusParams_CanOpMode
+
+
+class VectorCanFdParams(NamedTuple):
+    bitrate: int
+    data_bitrate: int
+    sjw_abr: int
+    tseg1_abr: int
+    tseg2_abr: int
+    sam_abr: int
+    sjw_dbr: int
+    tseg1_dbr: int
+    tseg2_dbr: int
+    output_mode: xldefine.XL_OutputMode
+    can_op_mode: xldefine.XL_CANFD_BusParams_CanOpMode
+
+
+class VectorBusParams(NamedTuple):
+    bus_type: xldefine.XL_BusTypes
+    can: VectorCanParams
+    canfd: VectorCanFdParams
+
+
 class VectorChannelConfig(NamedTuple):
+    """NamedTuple which contains the channel properties from Vector XL API."""
+
     name: str
-    hwType: xldefine.XL_HardwareType
-    hwIndex: int
-    hwChannel: int
-    channelIndex: int
-    channelMask: int
-    channelCapabilities: xldefine.XL_ChannelCapabilities
-    channelBusCapabilities: xldefine.XL_BusCapabilities
-    isOnBus: bool
-    connectedBusType: xldefine.XL_BusTypes
-    serialNumber: int
-    articleNumber: int
-    transceiverName: str
+    hw_type: xldefine.XL_HardwareType
+    hw_index: int
+    hw_channel: int
+    channel_index: int
+    channel_mask: int
+    channel_capabilities: xldefine.XL_ChannelCapabilities
+    channel_bus_capabilities: xldefine.XL_BusCapabilities
+    is_on_bus: bool
+    connected_bus_type: xldefine.XL_BusTypes
+    bus_params: VectorBusParams
+    serial_number: int
+    article_number: int
+    transceiver_name: str
+
+
+def _get_xl_driver_config() -> xlclass.XLdriverConfig:
+    if xldriver is None:
+        raise VectorError(
+            error_code=xldefine.XL_Status.XL_ERR_DLL_NOT_FOUND,
+            error_string="xldriver is unavailable",
+            function="_get_xl_driver_config",
+        )
+    driver_config = xlclass.XLdriverConfig()
+    xldriver.xlOpenDriver()
+    xldriver.xlGetDriverConfig(driver_config)
+    xldriver.xlCloseDriver()
+    return driver_config
+
+
+def _read_bus_params_from_c_struct(bus_params: xlclass.XLbusParams) -> VectorBusParams:
+    return VectorBusParams(
+        bus_type=xldefine.XL_BusTypes(bus_params.busType),
+        can=VectorCanParams(
+            bitrate=bus_params.data.can.bitRate,
+            sjw=bus_params.data.can.sjw,
+            tseg1=bus_params.data.can.tseg1,
+            tseg2=bus_params.data.can.tseg2,
+            sam=bus_params.data.can.sam,
+            output_mode=xldefine.XL_OutputMode(bus_params.data.can.outputMode),
+            can_op_mode=xldefine.XL_CANFD_BusParams_CanOpMode(
+                bus_params.data.can.canOpMode
+            ),
+        ),
+        canfd=VectorCanFdParams(
+            bitrate=bus_params.data.canFD.arbitrationBitRate,
+            data_bitrate=bus_params.data.canFD.dataBitRate,
+            sjw_abr=bus_params.data.canFD.sjwAbr,
+            tseg1_abr=bus_params.data.canFD.tseg1Abr,
+            tseg2_abr=bus_params.data.canFD.tseg2Abr,
+            sam_abr=bus_params.data.canFD.samAbr,
+            sjw_dbr=bus_params.data.canFD.sjwDbr,
+            tseg1_dbr=bus_params.data.canFD.tseg1Dbr,
+            tseg2_dbr=bus_params.data.canFD.tseg2Dbr,
+            output_mode=xldefine.XL_OutputMode(bus_params.data.canFD.outputMode),
+            can_op_mode=xldefine.XL_CANFD_BusParams_CanOpMode(
+                bus_params.data.canFD.canOpMode
+            ),
+        ),
+    )
 
 
 def get_channel_configs() -> List[VectorChannelConfig]:
-    if xldriver is None:
-        return []
-    driver_config = xlclass.XLdriverConfig()
+    """Read channel properties from Vector XL API."""
     try:
-        xldriver.xlOpenDriver()
-        xldriver.xlGetDriverConfig(driver_config)
-        xldriver.xlCloseDriver()
+        driver_config = _get_xl_driver_config()
     except VectorError:
-        pass
+        return []
 
     channel_list: List[VectorChannelConfig] = []
     for i in range(driver_config.channelCount):
         xlcc: xlclass.XLchannelConfig = driver_config.channel[i]
         vcc = VectorChannelConfig(
             name=xlcc.name.decode(),
-            hwType=xldefine.XL_HardwareType(xlcc.hwType),
-            hwIndex=xlcc.hwIndex,
-            hwChannel=xlcc.hwChannel,
-            channelIndex=xlcc.channelIndex,
-            channelMask=xlcc.channelMask,
-            channelCapabilities=xldefine.XL_ChannelCapabilities(
+            hw_type=xldefine.XL_HardwareType(xlcc.hwType),
+            hw_index=xlcc.hwIndex,
+            hw_channel=xlcc.hwChannel,
+            channel_index=xlcc.channelIndex,
+            channel_mask=xlcc.channelMask,
+            channel_capabilities=xldefine.XL_ChannelCapabilities(
                 xlcc.channelCapabilities
             ),
-            channelBusCapabilities=xldefine.XL_BusCapabilities(
+            channel_bus_capabilities=xldefine.XL_BusCapabilities(
                 xlcc.channelBusCapabilities
             ),
-            isOnBus=bool(xlcc.isOnBus),
-            connectedBusType=xldefine.XL_BusTypes(xlcc.connectedBusType),
-            serialNumber=xlcc.serialNumber,
-            articleNumber=xlcc.articleNumber,
-            transceiverName=xlcc.transceiverName.decode(),
+            is_on_bus=bool(xlcc.isOnBus),
+            bus_params=_read_bus_params_from_c_struct(xlcc.busParams),
+            connected_bus_type=xldefine.XL_BusTypes(xlcc.connectedBusType),
+            serial_number=xlcc.serialNumber,
+            article_number=xlcc.articleNumber,
+            transceiver_name=xlcc.transceiverName.decode(),
         )
         channel_list.append(vcc)
     return channel_list
