@@ -5,7 +5,7 @@ in the recorded order an time intervals.
 """
 import gzip
 import pathlib
-from time import time, sleep
+import time
 import typing
 
 from pkg_resources import iter_entry_points
@@ -16,6 +16,7 @@ from .blf import BLFReader
 from .canutils import CanutilsLogReader
 from .csv import CSVReader
 from .sqlite import SqliteReader
+from .trc import TRCReader
 from ..typechecking import StringPathLike, FileLike, AcceptedIOType
 from ..message import Message
 
@@ -30,6 +31,7 @@ class LogReader(MessageReader):
       * .csv
       * .db
       * .log
+      * .trc
 
     Gzip compressed files can be used as long as the original
     files suffix is one of the above (e.g. filename.asc.gz).
@@ -56,6 +58,7 @@ class LogReader(MessageReader):
         ".csv": CSVReader,
         ".db": SqliteReader,
         ".log": CanutilsLogReader,
+        ".trc": TRCReader,
     }
 
     @staticmethod
@@ -103,7 +106,7 @@ class LogReader(MessageReader):
         return real_suffix, gzip.open(filename, mode)
 
     def __iter__(self) -> typing.Generator[Message, None, None]:
-        pass
+        raise NotImplementedError()
 
 
 class MessageSync:  # pylint: disable=too-few-public-methods
@@ -132,8 +135,9 @@ class MessageSync:  # pylint: disable=too-few-public-methods
         self.skip = skip
 
     def __iter__(self) -> typing.Generator[Message, None, None]:
-        playback_start_time = time()
+        t_wakeup = playback_start_time = time.perf_counter()
         recorded_start_time = None
+        t_skipped = 0.0
 
         for message in self.raw_messages:
 
@@ -142,15 +146,19 @@ class MessageSync:  # pylint: disable=too-few-public-methods
                 if recorded_start_time is None:
                     recorded_start_time = message.timestamp
 
-                now = time()
-                current_offset = now - playback_start_time
-                recorded_offset_from_start = message.timestamp - recorded_start_time
-                remaining_gap = max(0.0, recorded_offset_from_start - current_offset)
-
-                sleep_period = max(self.gap, min(self.skip, remaining_gap))
+                t_wakeup = playback_start_time + (
+                    message.timestamp - t_skipped - recorded_start_time
+                )
             else:
-                sleep_period = self.gap
+                t_wakeup += self.gap
 
-            sleep(sleep_period)
+            sleep_period = t_wakeup - time.perf_counter()
+
+            if self.skip and sleep_period > self.skip:
+                t_skipped += sleep_period - self.skip
+                sleep_period = self.skip
+
+            if sleep_period > 1e-4:
+                time.sleep(sleep_period)
 
             yield message
