@@ -1,17 +1,11 @@
 #!/usr/bin/env python
-# coding: utf-8
 
-from __future__ import print_function
 
 import unittest
 import threading
-try:
-    import queue
-except ImportError:
-    import Queue as queue
 import random
-
 import logging
+
 logging.getLogger(__file__).setLevel(logging.WARNING)
 
 # make a random bool:
@@ -19,11 +13,9 @@ rbool = lambda: bool(round(random.random()))
 
 import can
 
-channel = 'vcan0'
-can.rc['interface'] = 'virtual'
+channel = "vcan0"
 
 
-@unittest.skipIf('interface' not in can.rc, "Need a CAN interface")
 class ControllerAreaNetworkTestCase(unittest.TestCase):
     """
     This test ensures that what messages go in to the bus is what comes out.
@@ -43,9 +35,19 @@ class ControllerAreaNetworkTestCase(unittest.TestCase):
     extended_flags = [rbool() for _ in range(num_messages)]
 
     ids = list(range(num_messages))
-    data = list(bytearray([random.randrange(0, 2 ** 8 - 1)
-                           for a in range(random.randrange(9))])
-                for b in range(num_messages))
+    data = list(
+        bytearray([random.randrange(0, 2**8 - 1) for a in range(random.randrange(9))])
+        for b in range(num_messages)
+    )
+
+    def setUp(self):
+        # Save all can.rc defaults
+        self._can_rc = can.rc
+        can.rc = {"interface": "virtual"}
+
+    def tearDown(self):
+        # Restore the defaults
+        can.rc = self._can_rc
 
     def producer(self, ready_event, msg_read):
         self.client_bus = can.interface.Bus(channel=channel)
@@ -55,10 +57,10 @@ class ControllerAreaNetworkTestCase(unittest.TestCase):
                 arbitration_id=self.ids[i],
                 is_remote_frame=self.remote_flags[i],
                 is_error_frame=self.error_flags[i],
-                extended_id=self.extended_flags[i],
-                data=self.data[i]
+                is_extended_id=self.extended_flags[i],
+                data=self.data[i],
             )
-            #logging.debug("writing message: {}".format(m))
+            # logging.debug("writing message: {}".format(m))
             if msg_read is not None:
                 # Don't send until the other thread is ready
                 msg_read.wait()
@@ -99,9 +101,9 @@ class ControllerAreaNetworkTestCase(unittest.TestCase):
             msg_read.set()
             msg = self.server_bus.recv(timeout=0.5)
             self.assertIsNotNone(msg, "Didn't receive a message")
-            #logging.debug("Received message {} with data: {}".format(i, msg.data))
+            # logging.debug("Received message {} with data: {}".format(i, msg.data))
 
-            self.assertEqual(msg.id_type, self.extended_flags[i])
+            self.assertEqual(msg.is_extended_id, self.extended_flags[i])
             if not msg.is_remote_frame:
                 self.assertEqual(msg.data, self.data[i])
             self.assertEqual(msg.arbitration_id, self.ids[i])
@@ -115,5 +117,6 @@ class ControllerAreaNetworkTestCase(unittest.TestCase):
         self.server_bus.flush_tx_buffer()
         self.server_bus.shutdown()
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     unittest.main()
