@@ -37,7 +37,7 @@ import pytest
 
 import can
 from can.viewer import CanViewer, parse_args
-
+from test.config import IS_CI
 
 # Allow the curses module to be missing (e.g. on PyPy on Windows)
 try:
@@ -251,9 +251,10 @@ class CanViewerTest(unittest.TestCase):
                     if _id["dt"] == 0:
                         self.assertEqual(_id["count"], 1)
                     else:
-                        self.assertTrue(
-                            pytest.approx(_id["dt"], 0.1)
-                        )  # dt should be ~0.1 s
+                        if not IS_CI:  # do not test timing in CI
+                            assert _id["dt"] == pytest.approx(
+                                0.1, abs=5e-2
+                            )  # dt should be ~0.1 s
                         self.assertEqual(_id["count"], 2)
                 else:
                     # Make sure dt is 0
@@ -347,7 +348,7 @@ class CanViewerTest(unittest.TestCase):
 
         raw_data = self.pack_data(CANOPEN_TPDO2 + 1, data_structs, 12.34, 4.5, 6)
         parsed_data = CanViewer.unpack_data(CANOPEN_TPDO2 + 1, data_structs, raw_data)
-        self.assertTrue(pytest.approx(parsed_data, [12.34, 4.5, 6]))
+        assert parsed_data == pytest.approx([12.34, 4.5, 6])
         self.assertTrue(
             isinstance(parsed_data[0], float)
             and isinstance(parsed_data[1], float)
@@ -356,14 +357,14 @@ class CanViewerTest(unittest.TestCase):
 
         raw_data = self.pack_data(CANOPEN_TPDO3 + 1, data_structs, 123.45, 67.89)
         parsed_data = CanViewer.unpack_data(CANOPEN_TPDO3 + 1, data_structs, raw_data)
-        self.assertTrue(pytest.approx(parsed_data, [123.45, 67.89]))
+        assert parsed_data == pytest.approx([123.45, 67.89])
         self.assertTrue(all(isinstance(d, float) for d in parsed_data))
 
         raw_data = self.pack_data(
             CANOPEN_TPDO4 + 1, data_structs, math.pi / 2.0, math.pi
         )
         parsed_data = CanViewer.unpack_data(CANOPEN_TPDO4 + 1, data_structs, raw_data)
-        self.assertTrue(pytest.approx(parsed_data, [math.pi / 2.0, math.pi]))
+        assert parsed_data == pytest.approx([math.pi / 2.0, math.pi])
         self.assertTrue(all(isinstance(d, float) for d in parsed_data))
 
         raw_data = self.pack_data(CANOPEN_TPDO1 + 2, data_structs)
